﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Diagnostics;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading;
using System.Windows.Forms;
using BASeCamp.Configuration;
using System.IO;
using Microsoft.SharePoint.Client;
using Microsoft.SharePoint.Client.Application;

namespace LinkerClicker
{
    class Program
    {
        static NotifyIcon ni = null;
        
        static INIFile ifile = null;
        static bool quitapp;
        static ContextMenuStrip rightclickmenu = null;
        static System.Threading.Timer tmr = null;
        static string minifile = Path.Combine(Environment.GetFolderPath(Environment.SpecialFolder.ApplicationData),"LinkClicker" + Path.DirectorySeparatorChar +  "LinkClicker.ini");
        static TimeSpan AuthDelay;
        static Uri ServerUri;
        static String loginName, loginpassword;
        static FormsAuthenticationLoginInfo authinfo;
        private static void reload()
        {
            ifile = new INIFile(minifile);
            //section Settings, item URL:
            String useUri = ifile["Settings"]["URL"].Value;
            if (useUri.Length == 0) ServerUri = null; else ServerUri = new Uri(useUri);
            //section Settings, item FollowText.
            loginName = ifile["Settings"]["username"].GetValue("username");
            loginpassword = ifile["Settings"]["password"].GetValue("password");
            //set delay to timespan in INI. default: 15 minutes.
            AuthDelay = new TimeSpan(0, 0, 0, ifile["Settings"]["AuthDelay"].GetValue(60*15));

            authinfo = new FormsAuthenticationLoginInfo(loginName, loginpassword);
            
        }
        static void CreateTimer()
        {
            //initializes the timer.
            //use AuthDelay, create a Timer that fires at that interval.
            if (tmr != null)
            {
                tmr.Dispose();
                tmr = null;
            }
            tmr = new System.Threading.Timer(TimerCall, null, new TimeSpan(), AuthDelay);
            TimerCall(null);
            
        }
        static void TimerCall(Object o)
        {
            //do the task of reauthentication.
            //use wb to navigate to the URL, and wait for it to complete.
            //we will use the sharepoint Client API.
            
            new Thread(() =>
            {
                try
                {
                    Microsoft.SharePoint.Client.Application.Authentication auth = new Authentication(ServerUri);
                    LoginResult lresult = auth.Login(loginName, loginpassword);
                    finishlogin(lresult);
                }
                catch (NullReferenceException)
                {


                    ni.ShowBalloonTip(10000, "Error", "invalid or missing URL in INI File.", ToolTipIcon.Error);
                }
                catch (Exception exx)
                {
                    ni.ShowBalloonTip(10000, "Error", "An Error occured attempted to reauthenticate the Sharepoint Server:\n" + exx.Message, ToolTipIcon.Error);



                }
            }).Start();
            
        }
        static void finishlogin(LoginResult resultitem)
        {
            //show notification balloon to indicate result.
            String usemessage,useTitle;
            ToolTipIcon selectedicon = ToolTipIcon.Info; 
            switch(resultitem.ErrorCode)
            {
                    //format string
                case LoginErrorCode.NoError:
                    selectedicon = ToolTipIcon.Info;
                    useTitle = "Sharepoint reauthenticated.";
                    usemessage = "reauthenticated as {0} on {1}.";
                    break;
                case LoginErrorCode.NotInFormsAuthenticationMode:
                    selectedicon=ToolTipIcon.Error;
                    useTitle = "reauthentication failure";
                    usemessage = "reauthentication as {0} on {1} failed; Server is not in Forms Authentication Mode.";
                    break;
                case LoginErrorCode.PasswordNotMatch:
                    selectedicon = ToolTipIcon.Error;
                    useTitle = "reauthentication failure";
                    usemessage="reauthentication as {0} on {1} failed. Username or password mismatch";
                    break;
                    
                default:
                    throw new ArgumentOutOfRangeException();
                    

            }
            String messageshow = String.Format(usemessage, loginName, ServerUri.ToString());
            ni.ShowBalloonTip(10000, useTitle, messageshow, selectedicon);

        }
        static MessageForm mf = new MessageForm();
        static void Main(string[] args)
        {
            mf.Visible = false;
            ni = new NotifyIcon();
            ni.Text = "Automatic Link Clicker";
            ni.Icon =   Icon.FromHandle(SPAuth.lockicon.GetHicon());
            ni.Visible = true;
            //open INI File.
            rightclickmenu = new ContextMenuStrip();
            rightclickmenu.Items.Add("Reauth Now").Click += ReAuth_Click;
            rightclickmenu.Items.Add("Exit").Click += Exit_Click;
            ni.ContextMenuStrip = rightclickmenu;
            
            reload();
            //create timer.
            CreateTimer();
            new Thread(() =>
            {
                while (!quitapp)
                {
                }
                Application.Exit();
            }).Start();
            Application.Run();


        }

        

        static void ReAuth_Click(object sender, EventArgs e)
        {
            TimerCall(null);
        }

        static void Exit_Click(object sender, EventArgs e)
        {
               //called when Exit menu item is clicked.
            //set flag to exit, the while loop in main will terminate and app will exit.
            quitapp = true; 
        }
    }
}
